@@ -1,5 +1,5 @@
 /*
-    iom_conf.h -- generic I/O multiplexor config
+    iom_conf.h -- generic I/O multiplexer config
     Copyright (C) 2003, 2004 Marc Lehmann <pcg@goof.com>
  
     This program is free software; you can redistribute it and/or modify
@@ -14,17 +14,18 @@
  
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc. 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc. 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #ifndef RXVT_IOM_CONF_H__
 #define RXVT_IOM_CONF_H__
 
-#include <rxvtstl.h>
+#include <rxvtutil.h>
 
 #define IOM_IO 1
 #define IOM_TIME 1
 #define IOM_CHECK 1
+#define IOM_SIG 1
 
 #endif
 
